@@ -286,40 +286,49 @@ void HeaderGenerator::writeTypeConverterDecl(QTextStream& s, const TypeEntry* ty
     }
 }
 
-void HeaderGenerator::writeTypeIndexDefineLine(QTextStream& s, const TypeEntry* typeEntry, int& idx)
+static void _writeTypeIndexDefineLine(QTextStream& s, const QString& variableName, int typeIndex)
+{
+    s << "#define ";
+    s.setFieldWidth(60);
+    s << variableName;
+    s.setFieldWidth(0);
+    s << ' ' << typeIndex << endl;
+}
+void HeaderGenerator::writeTypeIndexDefineLine(QTextStream& s, const TypeEntry* typeEntry)
 {
     if (!typeEntry || !typeEntry->generateCode())
         return;
     s.setFieldAlignment(QTextStream::AlignLeft);
-    s << "#define ";
-    s.setFieldWidth(60);
-    s << getTypeIndexVariableName(typeEntry);
-    s.setFieldWidth(0);
-    s << ' ' << (idx++) << endl;
+    int typeIndex = getTypeIndex(typeEntry);
+    _writeTypeIndexDefineLine(s, getTypeIndexVariableName(typeEntry), typeIndex);
+    if (typeEntry->isComplex()) {
+        const ComplexTypeEntry* cType = reinterpret_cast<const ComplexTypeEntry*>(typeEntry);
+        if (cType->baseContainerType()) {
+            const AbstractMetaClass* metaClass = classes().findClass(cType);
+            _writeTypeIndexDefineLine(s, getTypeIndexVariableName(metaClass, true), typeIndex);
+        }
+    }
     if (typeEntry->isEnum()) {
         const EnumTypeEntry* ete = reinterpret_cast<const EnumTypeEntry*>(typeEntry);
         if (ete->flags())
-            writeTypeIndexDefineLine(s, ete->flags(), idx);
+            writeTypeIndexDefineLine(s, ete->flags());
     }
 }
 
-void HeaderGenerator::writeTypeIndexDefine(QTextStream& s, const AbstractMetaClass* metaClass, int& idx)
+void HeaderGenerator::writeTypeIndexDefine(QTextStream& s, const AbstractMetaClass* metaClass)
 {
     if (!metaClass->typeEntry()->generateCode())
         return;
-    writeTypeIndexDefineLine(s, metaClass->typeEntry(), idx);
+    writeTypeIndexDefineLine(s, metaClass->typeEntry());
     foreach (const AbstractMetaEnum* metaEnum, metaClass->enums()) {
         if (metaEnum->isPrivate())
             continue;
-        writeTypeIndexDefineLine(s, metaEnum->typeEntry(), idx);
+        writeTypeIndexDefineLine(s, metaEnum->typeEntry());
     }
 }
 
 void HeaderGenerator::finishGeneration()
 {
-    if (classes().isEmpty())
-        return;
-
     // Generate the main header for this module.
     // This header should be included by binding modules
     // extendind on top of this one.
@@ -338,19 +347,18 @@ void HeaderGenerator::finishGeneration()
     Indentation indent(INDENT);
 
     macrosStream << "// Type indices" << endl;
-    int idx = 0;
     AbstractMetaEnumList globalEnums = this->globalEnums();
     foreach (const AbstractMetaClass* metaClass, classes()) {
-        writeTypeIndexDefine(macrosStream, metaClass, idx);
+        writeTypeIndexDefine(macrosStream, metaClass);
         lookForEnumsInClassesNotToBeGenerated(globalEnums, metaClass);
     }
     foreach (const AbstractMetaEnum* metaEnum, globalEnums)
-        writeTypeIndexDefineLine(macrosStream, metaEnum->typeEntry(), idx);
+        writeTypeIndexDefineLine(macrosStream, metaEnum->typeEntry());
     macrosStream << "#define ";
     macrosStream.setFieldWidth(60);
     macrosStream << "SBK_"+moduleName()+"_IDX_COUNT";
     macrosStream.setFieldWidth(0);
-    macrosStream << ' ' << idx << endl << endl;
+    macrosStream << ' ' << getMaxTypeIndex() << endl << endl;
     macrosStream << "// This variable stores all python types exported by this module" << endl;
     macrosStream << "extern PyTypeObject** " << cppApiVariableName() << ';' << endl << endl;
 
