@@ -24,12 +24,22 @@
 #ifndef SHIBOKENGENERATOR_H
 #define SHIBOKENGENERATOR_H
 
-#define PYTHON_RETURN_VAR         "pyResult"
-#define CPP_RETURN_VAR            "cppResult"
-#define THREAD_STATE_SAVER_VAR    "threadStateSaver"
-#define CPP_SELF_VAR              "cppSelf"
+#define CONV_RULE_OUT_VAR_SUFFIX  "_out"
 #define CPP_ARG                   "cppArg"
-#define CPP_ARG0                  (CPP_ARG "0")
+#define CPP_ARG0                  CPP_ARG"0"
+#define CPP_ARG_REMOVED           "removed_"CPP_ARG
+#define CPP_RETURN_VAR            "cppResult"
+#define CPP_SELF_VAR              "cppSelf"
+#define PYTHON_ARG                "pyArg"
+#define PYTHON_ARGS               PYTHON_ARG"s"
+#define PYTHON_OVERRIDE_VAR       "pyOverride"
+#define PYTHON_RETURN_VAR         "pyResult"
+#define PYTHON_SELF_VAR           "self"
+#define THREAD_STATE_SAVER_VAR    "threadStateSaver"
+#define BEGIN_ALLOW_THREADS       "PyThreadState* _save = PyEval_SaveThread(); // Py_BEGIN_ALLOW_THREADS"
+#define END_ALLOW_THREADS         "PyEval_RestoreThread(_save); // Py_END_ALLOW_THREADS"
+#define MIN_CTOR_ERROR_MSG        "Could not find a minimal constructor for type '%1'. "\
+                                  "This will result in a compilation error."
 
 #include <generator.h>
 #include <QtCore/QTextStream>
@@ -45,6 +55,7 @@ class ShibokenGenerator : public Generator
 {
 public:
     ShibokenGenerator();
+    virtual ~ShibokenGenerator();
 
     QString translateTypeForWrapperMethod(const AbstractMetaType* cType,
                                           const AbstractMetaClass* context, Options opt = NoOption) const;
@@ -103,26 +114,54 @@ public:
                                 const AbstractMetaFunction* func,
                                 Options options = NoOption) const;
     QString functionReturnType(const AbstractMetaFunction* func, Options options = NoOption) const;
-    /**
-     *   Write a code snip into the buffer \p s.
-     *   CodeSnip are codes inside inject-code tags.
-     *   \param s    the buffer
-     *   \param code_snips   a list of code snips
-     *   \param position     the position to insert the code snip
-     *   \param language     the kind of code snip
-     *   \param func the cpp function
-     *   \param lastArg last argument whose value is available, usually the last;
-     *                  a NULL pointer indicates that no argument will be available,
-     *                  i.e. a call without arguments.
-     *   \param context the class context for the place where the code snip will be written
-     */
-    void writeCodeSnips(QTextStream &s,
-                        const CodeSnipList &code_snips,
+
+    /// Utility function for writeCodeSnips.
+    typedef QPair<const AbstractMetaArgument*, QString> ArgumentVarReplacementPair;
+    typedef QList<ArgumentVarReplacementPair> ArgumentVarReplacementList;
+    ArgumentVarReplacementList getArgumentReplacement(const AbstractMetaFunction* func,
+                                                      bool usePyArgs, TypeSystem::Language language,
+                                                      const AbstractMetaArgument* lastArg);
+
+    /// Write user's custom code snippets at class or module level.
+    void writeCodeSnips(QTextStream& s,
+                        const CodeSnipList& codeSnips,
                         CodeSnip::Position position,
                         TypeSystem::Language language,
-                        const AbstractMetaFunction* func = 0,
-                        const AbstractMetaArgument* lastArg = 0,
                         const AbstractMetaClass* context = 0);
+    /// Write user's custom code snippets at function level.
+    void writeCodeSnips(QTextStream& s,
+                        const CodeSnipList& codeSnips,
+                        CodeSnip::Position position,
+                        TypeSystem::Language language,
+                        const AbstractMetaFunction* func,
+                        const AbstractMetaArgument* lastArg = 0);
+
+    /// Returns a string with the user's custom code snippets that comply with \p position and \p language.
+    QString getCodeSnippets(const CodeSnipList& codeSnips, CodeSnip::Position position, TypeSystem::Language language);
+
+    /// Replaces variables for the user's custom code at global or class level.
+    void processCodeSnip(QString& code, const AbstractMetaClass* context = 0);
+
+    /// Replaces the %CONVERTTOPYTHON type system variable.
+    inline void replaceConvertToPythonTypeSystemVariable(QString& code)
+    {
+        replaceConverterTypeSystemVariable(TypeSystemToPythonFunction, code);
+    }
+    /// Replaces the %CONVERTTOCPP type system variable.
+    inline void replaceConvertToCppTypeSystemVariable(QString& code)
+    {
+        replaceConverterTypeSystemVariable(TypeSystemToCppFunction, code);
+    }
+    /// Replaces the %ISCONVERTIBLE type system variable.
+    inline void replaceIsConvertibleToCppTypeSystemVariable(QString& code)
+    {
+        replaceConverterTypeSystemVariable(TypeSystemIsConvertibleFunction, code);
+    }
+    /// Replaces the %CHECKTYPE type system variable.
+    inline void replaceTypeCheckTypeSystemVariable(QString& code)
+    {
+        replaceConverterTypeSystemVariable(TypeSystemCheckFunction, code);
+    }
 
     /**
      *   Verifies if any of the function's code injections of the "target"
@@ -167,6 +206,12 @@ public:
      */
     bool injectedCodeHasReturnValueAttribution(const AbstractMetaFunction* func, TypeSystem::Language language = TypeSystem::TargetLangCode);
 
+    /**
+     *   Verifies if any of the function's code injections uses the type system variable
+     *   for function arguments of a given index.
+     */
+    bool injectedCodeUsesArgument(const AbstractMetaFunction* func, int argumentIndex);
+
     /**
      *   Function which parse the metafunction information
      *   \param func the function witch will be parserd
@@ -197,15 +242,17 @@ public:
     const AbstractMetaClass* getMultipleInheritingClass(const AbstractMetaClass* metaClass);
 
     void writeBaseConversion(QTextStream& s, const AbstractMetaType* type,
-                             const AbstractMetaClass* context, Options options = NoOption);
+                             const AbstractMetaClass* context = 0, Options options = NoOption);
     /// Simpler version of writeBaseConversion, uses only the base name of the type.
     void writeBaseConversion(QTextStream& s, const TypeEntry* type);
     void writeToPythonConversion(QTextStream& s, const AbstractMetaType* type,
-                                 const AbstractMetaClass* context, const QString& argumentName = QString());
-    void writeToCppConversion(QTextStream& s, const AbstractMetaType* type,
-                              const AbstractMetaClass* context, const QString& argumentName, Options options = NoOption);
+                                 const AbstractMetaClass* context, const QString& argumentName);
+    void writeToCppConversion(QTextStream& s, const AbstractMetaType* type, const AbstractMetaClass* context, const QString& argumentName);
     void writeToCppConversion(QTextStream& s, const AbstractMetaClass* metaClass, const QString& argumentName);
 
+    /// Returns true if the argument is a pointer that rejects NULL values.
+    static bool shouldRejectNullPointerArgument(const AbstractMetaFunction* func, int argIndex);
+
     /// Verifies if the class should have a C++ wrapper generated for it, instead of only a Python wrapper.
     bool shouldGenerateCppWrapper(const AbstractMetaClass* metaClass) const;
 
@@ -239,6 +286,20 @@ public:
     static bool isCString(const AbstractMetaType* type);
     static bool isPairContainer(const AbstractMetaType* type);
 
+    /**
+     *  Returns true if the type passed has a Python wrapper for it.
+     *  Although namespace has a Python wrapper, it's not considered a type.
+     */
+    static bool isWrapperType(const TypeEntry* type);
+    static bool isWrapperType(const ComplexTypeEntry* type);
+    static bool isWrapperType(const AbstractMetaType* metaType);
+
+    /**
+     *  Checks if the type is an Object/QObject or pointer to Value Type.
+     *  In other words, tells if the type is "T*" and T has a Python wrapper.
+     */
+    static bool isPointerToWrapperType(const AbstractMetaType* type);
+
     /// Checks if an argument type should be dereferenced by the Python method wrapper before calling the C++ method.
     static bool shouldDereferenceArgumentPointer(const AbstractMetaArgument* arg);
     /// Checks if a meta type should be dereferenced by the Python method wrapper passing it to C++.
@@ -252,9 +313,20 @@ public:
     QString cpythonTypeName(const AbstractMetaClass* metaClass);
     QString cpythonTypeName(const TypeEntry* type);
     QString cpythonTypeNameExt(const TypeEntry* type);
+    QString cpythonTypeNameExt(const AbstractMetaType* type);
     QString cpythonCheckFunction(const TypeEntry* type, bool genericNumberType = false);
     QString cpythonCheckFunction(const AbstractMetaType* metaType, bool genericNumberType = false);
-    QString guessCPythonCheckFunction(const QString& type);
+    /**
+     *  Receives the argument \p type and tries to find the appropriate AbstractMetaType for it
+     *  or a custom type check.
+     *  \param type     A string representing the type to be discovered.
+     *  \param metaType A pointer to an AbstractMetaType pointer, to where write a new meta type object
+     *                  if one is produced from the \p type string. This object must be deallocated by
+     *                  the caller. It will set the target variable to NULL, is \p type is a Python type.
+     *  \return A custom check if \p type is a custom type, or an empty string if \p metaType
+     *          receives an existing type object.
+     */
+    QString guessCPythonCheckFunction(const QString& type, AbstractMetaType** metaType);
     QString cpythonIsConvertibleFunction(const TypeEntry* type, bool genericNumberType = false, bool checkExact = false);
     QString cpythonIsConvertibleFunction(const AbstractMetaType* metaType, bool genericNumberType = false);
     QString cpythonIsConvertibleFunction(const AbstractMetaArgument* metaArg, bool genericNumberType = false)
@@ -262,6 +334,13 @@ public:
         return cpythonIsConvertibleFunction(metaArg->type(), genericNumberType);
     }
     QString guessCPythonIsConvertible(const QString& type);
+
+    QString cpythonToCppConversionFunction(const AbstractMetaClass* metaClass);
+    QString cpythonToCppConversionFunction(const AbstractMetaType* type, const AbstractMetaClass* context = 0);
+    QString cpythonToPythonConversionFunction(const AbstractMetaType* type, const AbstractMetaClass* context = 0);
+    QString cpythonToPythonConversionFunction(const AbstractMetaClass* metaClass);
+    QString cpythonToPythonConversionFunction(const TypeEntry* type);
+
     QString cpythonFunctionName(const AbstractMetaFunction* func);
     QString cpythonMethodDefinitionName(const AbstractMetaFunction* func);
     QString cpythonGettersSettersDefinitionName(const AbstractMetaClass* metaClass);
@@ -269,7 +348,7 @@ public:
     QString cpythonSetattroFunctionName(const AbstractMetaClass* metaClass);
     QString cpythonGetterFunctionName(const AbstractMetaField* metaField);
     QString cpythonSetterFunctionName(const AbstractMetaField* metaField);
-    QString cpythonWrapperCPtr(const AbstractMetaClass* metaClass, QString argName = "self");
+    QString cpythonWrapperCPtr(const AbstractMetaClass* metaClass, QString argName = PYTHON_SELF_VAR);
     QString cpythonWrapperCPtr(const AbstractMetaType* metaType, QString argName);
     QString cpythonWrapperCPtr(const TypeEntry* type, QString argName);
 
@@ -315,23 +394,42 @@ public:
     /// Returns true if the generated code should use the "#define protected public" hack.
     bool avoidProtectedHack() const;
     QString cppApiVariableName(const QString& moduleName = QString()) const;
-    QString getTypeIndexVariableName(const TypeEntry* metaType);
+    /**
+     *  Returns the type index variable name for a given class. If \p alternativeTemplateName is true
+     *  and the class is a typedef for a template class instantiation, it will return an alternative name
+     *  made of the template class and the instantiation values, or an empty string if the class isn't
+     *  derived from a template class at all.
+     */
+    QString getTypeIndexVariableName(const AbstractMetaClass* metaClass, bool alternativeTemplateName = false);
+    QString getTypeIndexVariableName(const TypeEntry* type);
+    QString getTypeIndexVariableName(const AbstractMetaType* type);
+
+    /// Returns the proper full name for \p type.
+    QString getFullTypeName(const TypeEntry* type);
+    QString getFullTypeName(const AbstractMetaType* type);
+    QString getFullTypeName(const AbstractMetaClass* metaClass);
+
     /// Returns true if the user don't want verbose error messages on the generated bindings.
     bool verboseErrorMessagesDisabled() const;
 
     /**
      *   Builds an AbstractMetaType object from a QString.
      *   Returns NULL if no type could be built from the string.
-     *   \param typeString The string describing the type to be built.
+     *   \param typeSignature The string describing the type to be built.
      *   \return A new AbstractMetaType object that must be deleted by the caller, or a NULL pointer in case of failure.
      */
-    AbstractMetaType* buildAbstractMetaTypeFromString(QString typeString);
+    AbstractMetaType* buildAbstractMetaTypeFromString(QString typeSignature);
 
     /**
      *  Helper function to return the flags to be used by a meta type when
      * it needs to write some converter code.
      */
     static Options getConverterOptions(const AbstractMetaType* metaType);
+
+    /**
+     * Helper function to find for argument default value
+     */
+    static QString getDefaultValue(const AbstractMetaFunction* func, const AbstractMetaArgument* arg);
 protected:
     bool doSetup(const QMap<QString, QString>& args);
     // verify whether the class is copyable
@@ -342,20 +440,23 @@ protected:
     static QHash<QString, QString> m_pythonOperators;
     static QHash<QString, QString> m_formatUnits;
     static QHash<QString, QString> m_tpFuncs;
+    static QStringList m_knownPythonTypes;
 
     void clearTpFuncs();
 
     const char* name() const { return "Shiboken"; }
 
-    /**
-     *   Initialize correspondences between primitive and Python types
-     */
+    /// Initializes correspondences between primitive and Python types.
     static void initPrimitiveTypesCorrespondences();
+    /// Initializes a list of Python known type names.
+    static void initKnownPythonTypes();
 
     void writeFunctionCall(QTextStream& s,
                            const AbstractMetaFunction* metaFunc,
                            Options options = NoOption) const;
 
+    void writeUnusedVariableCast(QTextStream& s, const QString& variableName);
+
     AbstractMetaFunctionList filterFunctions(const AbstractMetaClass* metaClass);
 
     // All data about extended converters: the type entries of the target type, and a
@@ -368,6 +469,16 @@ protected:
     static bool pythonFunctionWrapperUsesListOfArguments(const OverloadData& overloadData);
 
     Indentor INDENT;
+
+    enum TypeSystemConverterVariable {
+        TypeSystemCheckFunction = 0,
+        TypeSystemIsConvertibleFunction,
+        TypeSystemToCppFunction,
+        TypeSystemToPythonFunction,
+        TypeSystemConverterVariables
+    };
+    void replaceConverterTypeSystemVariable(TypeSystemConverterVariable converterVariable, QString& code);
+
 private:
     bool m_useCtorHeuristic;
     bool m_userReturnValueHeuristic;
@@ -375,8 +486,13 @@ private:
     bool m_verboseErrorMessagesDisabled;
     bool m_useIsNullAsNbNonZero;
     bool m_avoidProtectedHack;
+
+    typedef QHash<QString, AbstractMetaType*> AbstractMetaTypeCache;
+    AbstractMetaTypeCache m_metaTypeFromStringCache;
+
+    /// Type system converter variable replacement names and regular expressions.
+    QString m_typeSystemConvName[TypeSystemConverterVariables];
+    QRegExp m_typeSystemConvRegEx[TypeSystemConverterVariables];
 };
 
-
 #endif // SHIBOKENGENERATOR_H
-
