@@ -183,13 +183,16 @@ void SbkDeallocWrapper(PyObject* pyObj)
             Shiboken::DtorCallerVisitor visitor(sbkObj);
             Shiboken::walkThroughClassHierarchy(pyObj->ob_type, &visitor);
         } else {
+            void* cptr = sbkObj->d->cptr[0];
+            Shiboken::Object::deallocData(sbkObj, true);
+
             Shiboken::ThreadStateSaver threadSaver;
             threadSaver.save();
-            sbkType->d->cpp_dtor(sbkObj->d->cptr[0]);
+            sbkType->d->cpp_dtor(cptr);
         }
+    } else {
+        Shiboken::Object::deallocData(sbkObj, true);
     }
-    //Always destroy object data during the python object destruction
-    Shiboken::Object::deallocData(sbkObj, true);
 }
 
 void SbkDeallocWrapperWithPrivateDtor(PyObject* self)
@@ -207,14 +210,16 @@ void SbkObjectTypeDealloc(PyObject* pyObj)
 
     PyObject_GC_UnTrack(pyObj);
     Py_TRASHCAN_SAFE_BEGIN(pyObj);
-    if(sbkType->d->user_data && sbkType->d->d_func) {
-        sbkType->d->d_func(sbkType->d->user_data);
-        sbkType->d->user_data = 0;
+    if (sbkType->d) {
+        if(sbkType->d->user_data && sbkType->d->d_func) {
+            sbkType->d->d_func(sbkType->d->user_data);
+            sbkType->d->user_data = 0;
+        }
+        free(sbkType->d->original_name);
+        sbkType->d->original_name = 0;
+        delete sbkType->d;
+        sbkType->d = 0;
     }
-    free(sbkType->d->original_name);
-    sbkType->d->original_name = 0;
-    delete sbkType->d;
-    sbkType->d = 0;
     Py_TRASHCAN_SAFE_END(pyObj);
 }
 
@@ -245,7 +250,6 @@ PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* k
 
     // The meta type creates a new type when the Python programmer extends a wrapped C++ class.
     SbkObjectType* newType = reinterpret_cast<SbkObjectType*>(PyType_Type.tp_new(metatype, args, kwds));
-
     if (!newType)
         return 0;
 
@@ -321,7 +325,7 @@ namespace Shiboken
 
 static void decRefPyObjectList(const std::list<PyObject*> &pyObj, PyObject* skip = 0);
 
-void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visitor)
+static void _walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visitor)
 {
     PyObject* bases = currentType->tp_bases;
     Py_ssize_t numBases = PyTuple_GET_SIZE(bases);
@@ -333,23 +337,33 @@ void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visi
         } else {
             SbkObjectType* sbkType = reinterpret_cast<SbkObjectType*>(type);
             if (sbkType->d->is_user_type)
-                walkThroughClassHierarchy(type, visitor);
+                _walkThroughClassHierarchy(type, visitor);
             else
                 visitor->visit(sbkType);
         }
         if (visitor->wasFinished())
-            return;
+            break;
     }
 }
 
+void walkThroughClassHierarchy(PyTypeObject* currentType, HierarchyVisitor* visitor)
+{
+    _walkThroughClassHierarchy(currentType, visitor);
+    visitor->done();
+}
+
+
 bool importModule(const char* moduleName, PyTypeObject*** cppApiPtr)
 {
     PyObject* sysModules = PyImport_GetModuleDict();
     PyObject* module = PyDict_GetItemString(sysModules, moduleName);
-    if (!module)
+    if (!module) {
         module = PyImport_ImportModule(moduleName);
-    else
+        if (!module)
+            return false;
+    } else {
         Py_INCREF(module);
+    }
 
     Shiboken::AutoDecRef cppApi(PyObject_GetAttrString(module, "_Cpp_Api"));
     Py_DECREF(module);
@@ -367,18 +381,31 @@ bool importModule(const char* moduleName, PyTypeObject*** cppApiPtr)
 
 void DtorCallerVisitor::visit(SbkObjectType* node)
 {
-    Shiboken::ThreadStateSaver threadSaver;
-    threadSaver.save();
-    node->d->cpp_dtor(m_pyObj->d->cptr[m_count]);
-    m_count++;
+    m_ptrs.push_back(std::make_pair(m_pyObj->d->cptr[m_ptrs.size()], node));
 }
 
+void DtorCallerVisitor::done()
+{
+    Shiboken::Object::deallocData(m_pyObj, true);
+
+    std::list<std::pair<void*, SbkObjectType*> >::const_iterator it = m_ptrs.begin();
+    for (; it != m_ptrs.end(); ++it) {
+        Shiboken::ThreadStateSaver threadSaver;
+        threadSaver.save();
+        it->second->d->cpp_dtor(it->first);
+    }
+}
+
+namespace Module { void init(); }
+
 void init()
 {
     static bool shibokenAlreadInitialised = false;
     if (shibokenAlreadInitialised)
         return;
 
+    Module::init();
+
     initTypeResolver();
     PyEval_InitThreads();
 
@@ -533,19 +560,19 @@ bool isExternalConvertible(SbkObjectType* self, PyObject* obj)
     return self->d->ext_isconvertible && self->d->ext_isconvertible(obj);
 }
 
-bool hasCast(SbkObjectType* self)
+bool hasCast(SbkObjectType* type)
 {
-    return self->d->mi_specialcast;
+    return type->d->mi_specialcast;
 }
 
-void* cast(SbkObjectType* self, SbkObject* obj, PyTypeObject *target)
+void* cast(SbkObjectType* sourceType, SbkObject* obj, PyTypeObject* targetType)
 {
-    return self->d->mi_specialcast(Object::cppPointer(obj, target), reinterpret_cast<SbkObjectType*>(target));
+    return sourceType->d->mi_specialcast(Object::cppPointer(obj, targetType), reinterpret_cast<SbkObjectType*>(targetType));
 }
 
-void setCastFunction(SbkObjectType* self, SpecialCastFunction func)
+void setCastFunction(SbkObjectType* type, SpecialCastFunction func)
 {
-    self->d->mi_specialcast = func;
+    type->d->mi_specialcast = func;
 }
 
 void setOriginalName(SbkObjectType* self, const char* name)
@@ -598,6 +625,38 @@ void initPrivateData(SbkObjectType* self)
     memset(self->d, 0, sizeof(SbkObjectTypePrivate));
 }
 
+bool introduceWrapperType(PyObject* enclosingObject,
+                          const char* typeName, const char* originalName,
+                          SbkObjectType* type, ObjectDestructor cppObjDtor,
+                          SbkObjectType* baseType, PyObject* baseTypes,
+                          bool isInnerClass)
+{
+    initPrivateData(type);
+    setOriginalName(type, originalName);
+    setDestructorFunction(type, cppObjDtor);
+
+    if (baseType) {
+        type->super.ht_type.tp_base = (PyTypeObject*)baseType;
+        if (baseTypes) {
+            for (int i = 0; i < PySequence_Fast_GET_SIZE(baseTypes); ++i)
+                BindingManager::instance().addClassInheritance((SbkObjectType*)PySequence_Fast_GET_ITEM(baseTypes, i), type);
+            type->super.ht_type.tp_bases = baseTypes;
+        } else {
+            BindingManager::instance().addClassInheritance(baseType, type);
+        }
+    }
+
+    if (PyType_Ready((PyTypeObject*)type) < 0)
+        return false;
+
+    if (isInnerClass)
+        return PyDict_SetItemString(enclosingObject, typeName, (PyObject*)type) == 0;
+
+    //PyModule_AddObject steals type's reference.
+    Py_INCREF((PyObject*)type);
+    return PyModule_AddObject(enclosingObject, typeName, (PyObject*)type) == 0;
+}
+
 void setSubTypeInitHook(SbkObjectType* self, SubTypeInitHook func)
 {
     self->d->subtype_init = func;
@@ -705,7 +764,8 @@ void getOwnership(SbkObject* self)
 
 void getOwnership(PyObject* pyObj)
 {
-    setSequenceOwnership(pyObj, true);
+    if (pyObj)
+        setSequenceOwnership(pyObj, true);
 }
 
 void releaseOwnership(SbkObject* self)
@@ -794,7 +854,9 @@ void* cppPointer(SbkObject* pyObj, PyTypeObject* desiredType)
     int idx = 0;
     if (reinterpret_cast<SbkObjectType*>(type)->d->is_multicpp)
         idx = getTypeIndexOnHierarchy(type, desiredType);
-    return pyObj->d->cptr[idx];
+    if (pyObj->d->cptr)
+        return pyObj->d->cptr[idx];
+    return 0;
 }
 
 bool setCppPointer(SbkObject* sbkObj, PyTypeObject* desiredType, void* cptr)
@@ -930,6 +992,11 @@ void destroy(SbkObject* self, void* cppData)
     if (cppData && Shiboken::BindingManager::instance().hasWrapper(cppData)) {
         // Remove from BindingManager
         Shiboken::BindingManager::instance().releaseWrapper(self);
+        self->d->hasOwnership = false;
+
+        // the cpp object instance was deleted
+        delete[] self->d->cptr;
+        self->d->cptr = 0;
     }
 
     // After this point the object can be death do not use the self pointer bellow
@@ -939,6 +1006,11 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
 {
     ParentInfo* pInfo = child->d->parentInfo;
     if (!pInfo || !pInfo->parent) {
+        if (pInfo && pInfo->hasWrapperRef) {
+            pInfo->hasWrapperRef = false;
+            delete pInfo;
+            child->d->parentInfo = 0;
+        }
         return;
     }
     ChildrenList& oldBrothers = pInfo->parent->d->parentInfo->children;
@@ -952,8 +1024,15 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
     pInfo->parent = 0;
 
     // This will keep the wrapper reference, will wait for wrapper destruction to remove that
-    if (keepReference && child->d->containsCppWrapper)
+    if (keepReference &&
+        child->d->containsCppWrapper) {
+        //If have already a extra ref remove this one
+        if (pInfo->hasWrapperRef)
+            Py_CLEAR(child);
+        else
+            pInfo->hasWrapperRef = true;
         return;
+    }
 
     // Transfer ownership back to Python
     child->d->hasOwnership = giveOwnershipBack;
@@ -1043,13 +1122,14 @@ void deallocData(SbkObject* self, bool cleanup)
         clearReferences(self);
     }
 
-    // Remove from BindingManager
-    Shiboken::BindingManager::instance().releaseWrapper(self);
-
+    if (self->d->cptr) {
+        // Remove from BindingManager
+        Shiboken::BindingManager::instance().releaseWrapper(self);
+        delete[] self->d->cptr;
+        self->d->cptr = 0;
+        delete self->d;
+    }
     Py_XDECREF(self->ob_dict);
-    delete[] self->d->cptr;
-    self->d->cptr = 0;
-    delete self->d;
     Py_TYPE(self)->tp_free(self);
 }
 
@@ -1137,5 +1217,3 @@ void clearReferences(SbkObject* self)
 } // namespace Object
 
 } // namespace Shiboken
-
-
