@@ -1,5 +1,6 @@
 #include "abstract.h"
 #include "blackbox.h"
+#include "bytearray.h"
 #include "bucket.h"
 #include "collector.h"
 #include "complex.h"
@@ -23,14 +24,17 @@
 #include "nondefaultctor.h"
 #include "objectmodel.h"
 #include "objecttype.h"
+#include "objecttypeholder.h"
 #include "objecttypelayout.h"
 #include "objecttypereference.h"
 #include "objecttypeoperators.h"
 #include "objectview.h"
 #include "oddbool.h"
+#include "onlycopy.h"
 #include "overload.h"
 #include "pairuser.h"
 #include "pen.h"
+#include "photon.h"
 #include "point.h"
 #include "pointf.h"
 #include "pointerholder.h"
@@ -41,6 +45,7 @@
 #include "rect.h"
 #include "reference.h"
 #include "removednamespaces.h"
+#include "sample.h"
 #include "samplenamespace.h"
 #include "simplefile.h"
 #include "size.h"
